@@ -34,11 +34,10 @@ use parent 'LyricFinder::_Class';
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-our $VERSION = '1.11';
-our @FETCHERS = ();
+our $VERSION = '1.20';
 our $DEBUG = 0;  # If you want debug messages, set debug to a true value
 
-my @supported_mods = (qw(ApiLyricsOvh AZLyrics Genius Letras Musixmatch Cache));
+my @supported_mods = (qw(Cache ApiLyricsOvh AZLyrics Genius Letras Musixmatch));
 
 my %haveit;
 
@@ -46,15 +45,41 @@ foreach my $module (@supported_mods)
 {
 	$haveit{$module} = 0;
 	eval "use LyricFinder::$module; \$haveit{$module} = 1; 1";
-	push @FETCHERS, $module  if ($haveit{$module} && $module ne 'Cache');
 }
 
 sub new
 {
 	my $class = shift;
 
-	my $self = $class->SUPER::new('', @_);
-	@{$self->{'_fetchers'}} = @FETCHERS;
+	#EXTRACT ANY MAIN-SPECIFIC ARGUMENTS (NOT TO BE PASSED TO SUBMODULES):
+	my @args = ();
+	while (@_) {
+		my $arg = shift(@_);
+		if ($arg =~ /^\-omit$/o) {   #ALLOW USER TO OMIT SPECIFIC INSTALLED SUBMODULE(S):
+			my $omit = shift(@_);
+			my @omitModules = ref($omit) ? @{$omit} : split(/\,\s*/, $omit);
+			foreach my $omit (@omitModules)
+			{
+				$haveit{$omit} = 0  if (defined($haveit{$omit}) && $haveit{$omit});
+			}
+		} else {
+			push @args, $arg;
+		}
+	}
+
+	my $self = $class->SUPER::new('', @args);
+#	@{$self->{'_fetchers'}} = @FETCHERS;
+	@{$self->{'_fetchers'}} = ();
+	@{$self->{'_FETCHERS'}} = ();
+	#NOTE:  UPPER CASE _FETCHERS USED FOR "random" AND "all", & *NEVER* INCLUDES CACHE (1ST SUBMODULE TRIES CACHE)!:
+	#LOWER CASE _fetchers INCLUDES CACHE FIRST IF CACHE DIRECTORY AND IT'S NOT WRITEONLY, AS THIS IS FOR ORDER/TRIED LIST!
+	foreach my $module (@supported_mods)
+	{
+		next  unless ($haveit{$module} && $module ne 'Cache');
+		push @{$self->{'_FETCHERS'}}, $module;
+		push @{$self->{'_fetchers'}}, $module;
+	}
+	
 	unshift(@{$self->{'_fetchers'}}, 'Cache')  if ($haveit{'Cache'}
 			&& $self->{'-cache'} && $self->{'-cache'} !~ /^\>/);
 
@@ -117,6 +142,7 @@ sub _fetch {
 			if (defined($lyrics) && $lyrics =~ /\S/o) {
 				$self->{'Source'} = $finderModule->source();;
 				$self->{'Site'} = $finderModule->site();
+				$self->{'image_url'} = $finderModule->image_url();
 				@{$self->{'Credits'}} = $finderModule->credits();
 				$self->{'Tried'} =~ s/\,$//;
 				$self->_debug("i:Lyrics fetched from: ".$self->{'Source'});
@@ -139,31 +165,31 @@ sub fetch {
 
 	my @tryfetchers = ();
 
-	$self->_debug("Mod::fetch($artist, $title, $fetcherspec)!");
+	$self->_debug("LyricFinder::fetch($artist, $title, $fetcherspec)!");
 	$self->{'Tried'} = '';
 
 	$fetcherspec = 'random'  unless (defined($fetcherspec) && $fetcherspec);
 	$self->{'Source'} = 'none';
 	if ( $fetcherspec && !ref $fetcherspec && $fetcherspec !~ m'^auto$'i) {
 		# we've been given a specific fetcher to use:
-		if (grep /$fetcherspec/, @FETCHERS) {
+		if (grep /$fetcherspec/, @{$self->{'_FETCHERS'}}) {
 			push @tryfetchers, $fetcherspec;
 		} elsif ($fetcherspec =~ m'^random$'i) {
 			my $random_fetcher;
 			my %usedSources = ();
 			my $usedcnt = 0;
-			while ($usedcnt <= $#FETCHERS) {
-				$random_fetcher = int(rand(scalar @FETCHERS));
-				unless ($usedSources{$FETCHERS[$random_fetcher]}) {
-					push @tryfetchers, $FETCHERS[$random_fetcher];
-					$usedSources{$FETCHERS[$random_fetcher]} = 1;
+			while ($usedcnt <= $#{$self->{'_FETCHERS'}}) {
+				$random_fetcher = int(rand(scalar @{$self->{'_FETCHERS'}}));
+				unless ($usedSources{${$self->{'_FETCHERS'}}[$random_fetcher]}) {
+					push @tryfetchers, ${$self->{'_FETCHERS'}}[$random_fetcher];
+					$usedSources{${$self->{'_FETCHERS'}}[$random_fetcher]} = 1;
 					$usedcnt++;
 				}
 			}
 		} elsif ($fetcherspec =~ m'^Cache$'i && $haveit{'Cache'}) {
 			@tryfetchers = ('Cache');
 		} elsif ($fetcherspec =~ m'^All$'i) {
-			push @tryfetchers, @FETCHERS;
+			push @tryfetchers, @{$self->{'_FETCHERS'}};
 		} else { 
 			carp($self->{'Error'} = "s:Source (module) $fetcherspec isn't installed or is invalid!");
 			return;
@@ -171,7 +197,7 @@ sub fetch {
 	} elsif (ref $fetcherspec eq 'ARRAY') {
 		# we've got an arrayref of fetchers to use:
 		for my $fetcher (@$fetcherspec) {
-			if (grep /$fetcher/, @FETCHERS) {
+			if (grep /$fetcher/, @{$self->{'_FETCHERS'}}) {
 				push @tryfetchers, $fetcher;
 			} else {
 				carp("e:$fetcher isn't a valid fetcher, ignoring");
@@ -179,7 +205,7 @@ sub fetch {
 		}
 	} else {  #really shouldn't end up here (since default=random now), but leaving in for now.
 		# OK, try all available fetchers.
-		push @tryfetchers, @FETCHERS;
+		push @tryfetchers, @{$self->{'_FETCHERS'}};
 	}
 	$self->{'Order'} = join(',', @tryfetchers);
 	$#tryfetchers = $limit - 1  if (defined($limit) && $limit > 0 && $limit < scalar(@tryfetchers));
@@ -360,10 +386,11 @@ be I<1> ("I<true>").  The currently-supported options are:
 =item B<-agent> => I<"user-agent string">
 
 Specifies an alternate "user-agent" string sent to the lyric sites when 
-attempting to fetch lyrics.  Set the desired user-agent (ie. browser name) to 
-pass to the lyrics sites.  Some sites are pickey about receiving a user-agent 
-string that corresponds to a valid / supported web-browser to prevent their 
-sites from being "scraped" by programs, such as this.
+attempting to fetch lyrics.  Get / set the desired user-agent 
+(ie. browser name) to pass to the lyrics sites.  Some sites are pickey about 
+receiving a user-agent string that corresponds to a valid / supported 
+web-browser to prevent their sites from being "scraped" by programs, such 
+as this.
 
 Default:  I<"Mozilla/5.0 (X11; Linux x86_64; rv:84.0) Gecko/20100101 Firefox/84.0">.
 
@@ -413,9 +440,19 @@ one level of debug verbosity.
 
 Some sites (currently, only Musixmatch) may append some additional 
 information below the lyrics text.  If specified or given a I<true> value, 
-this will be suppressed and only the actual long lyrics returned.    
+this will be suppressed and only the actual song lyrics returned.    
 Default I<0> (false) - show any additional site-specific information.
 
+=item B<-omit> => I<"site-module[,site-module2...]]>
+
+Permits omitting specific sites which are currently installed from being 
+searched (namely when using I<random> or I<all>).  For example, to 
+exclude the Musixmatch site, specify:  I<-omit> => I<"Musixmatch">, which 
+will cause LyricFinder::Musixmatch to not be considered for lyrics search.  
+Default is for all installed sites (submodules) to be considered.  
+NOTE:  The site list can be specified as a comma-separated string OR as 
+an array reference, ie. I<-omit => [ qw(Musixmatch Genius) ]>.
+
 =item B<-site-module-name> => { I<"-option"> => I<value> [, I<"-option"> => I<value> ... ] }
 
 Specifies options for a specific site fetcher module.  These values 
@@ -524,6 +561,13 @@ Genius, Letras, and Musixmatch (NOTE the "x" in the spelling of "Musixmatch")!
 Returns lyrics as a string (includes line-breaks appropriate for the user's 
 operating system), or an empty string, if no lyrics found.
 
+=item I<$scalar> = $finder->B<image_url>()
+
+Returns a URL for a cover-art image, if one found on the lyrics page.  
+Currently, only the LyricFinder::Genius and LyricFinder::Musixmatch 
+sites contain cover-art images.  For the other sites, or if no image 
+is found, an empty string will be returned if this method is called.
+
 =item I<$scalar> = $finder->B<message>()
 
 Returns the last error string generated, or "Ok" if all's well.
