@@ -27,6 +27,7 @@ sub new
 	$self->{'Order'}   = '';
 	$self->{'Tried'}   = '';
 	$self->{'Url'}     = '';
+	$self->{'image_url'} = '';
 	$self->{'Credits'} = [];
 
 	#EXTRACT ANY ARGUMENTS:
@@ -123,6 +124,10 @@ sub cache {
 	}
 }
 
+sub image_url {
+	return shift->{'image_url'};
+}
+
 sub _check_inputs {
 	my $self = shift;
 
